package com.agiledeveloper;

import akka.actor.ActorRef;
import akka.actor.ActorSystem;
import akka.actor.Props;

public class CountPrimes {
  static final int NUMBER = 10000000;

  public static void main(String[] args) {
    computeSequential(NUMBER);
    computeConcurrent(NUMBER);
  }

  private static void computeConcurrent(final int number) {
    final int numberOfActors = Runtime.getRuntime().availableProcessors();

    ActorSystem actorSystem = ActorSystem.create();
    final ActorRef countAccumulator = actorSystem.actorOf(new Props(CountAccoumulator.class));
    countAccumulator.tell(new ExpectedMessages(numberOfActors), null);

    for(int i = 0; i < numberOfActors; i++) {
      ActorRef primeCounter = actorSystem.actorOf(new Props(PrimeCounter.class));
      final int lower = NUMBER / numberOfActors * i + 1;
      final int upper = lower + NUMBER / numberOfActors - 1;

      primeCounter.tell(new NumberRange(lower, upper), countAccumulator);
    }
  }

  private static void computeSequential(final int number) {
    final long start = System.nanoTime();
    final int count = Util.countPrimesBetween(1, number);
    final long end = System.nanoTime();
    System.out.println("Count: " + count);
    System.out.println("Time taken: " + (end - start)/1.0e9);
  }
}
